clear; clc;  n = 150;  S = 10000;  M = 80;

cr_v_J = chi2inv(0.95, 2);  cr_v_C = chi2inv(0.95, 1); 

%          [ sigma    v    gamma     p      sigma_d  ] 
nuisance = [ .02,    .07,    4,     .005,      .15   ];

sigma    = nuisance(1);     v  = nuisance(2);    

gamma    = nuisance(3);     p  = nuisance(4);

sigma_d  = nuisance(5);

solvetheta = @(tt)(gamma*sigma^2-sigma^2/2+(p+tt*gamma)*(exp(gamma*v)-p/(p+tt)*exp(gamma*v-v))-p*(v+tt/(p+tt*gamma)));
theta0     = fzero(@(tt)(solvetheta(tt)-6/100), 0);


theta1_1   = fzero(@(tt)(solvetheta(tt)-3/100),  0);
theta1_2   = fzero(@(tt)(solvetheta(tt)-9/100),  0);
theta2_1   = fzero(@(tt)(solvetheta(tt)-2/100),  0);
theta2_2   = fzero(@(tt)(solvetheta(tt)-10/100), 0);

theta_spac = [theta1_1, theta1_2; theta2_1, theta2_2];

eta        = linspace(-0.13, 0.13, M)';
eta        = [eta(1:M/2);0;eta(M/2+1:M)];
M          = M + 1;
para       = [theta0*ones(M,1), eta];

for b  = [1, 2]
    
tseed   = 0; randn('seed', tseed);
    
tic; 
     
theta_g = linspace(theta_spac(b,1), theta_spac(b,2), 2000);

J_test  = zeros(M,1);  C_test = zeros(M,1);  CC_test = zeros(M,1);

  for m = 1:M

  para_m     = para(m,:);
    
  J_test_m   = zeros(S,1); C_test_m   = zeros(S,1); CC_test_m  = zeros(S,1);

    parfor s = 1:S

    Data       = SData(nuisance, para_m, n); 

    [est_theta, omg, inv_omg, pval_J, G, J, mean_g, m0_1, m0_2, W0_theta] = gmm_cue(Data, nuisance, theta_g);

    [pval, CC_test_rst, C] = conditional_inference(Data, omg, inv_omg, G, J, mean_g, m0_1, m0_2, W0_theta);

    if J > cr_v_J; J_test_m(s,1) = 1; end

    if C > cr_v_C; C_test_m(s,1) = 1; end

    CC_test_m(s,1) = CC_test_rst;

    end

  J_test(m,1) = mean(J_test_m); C_test(m,1) = mean(C_test_m); CC_test(m,1) = mean(CC_test_m);

  end

time_cost = toc;

name = 'DR_J_and_CC_%d.mat';

save(sprintf(name,b),'J_test','C_test','CC_test','time_cost');  

end


% Auxiliary functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%        Function used to conduct the conditional specification test
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [pval, test, C] = conditional_inference(Data, omg, inv_omg, G, J, m_theta0, m0_1, m0_2, W0_theta)
%--------------------------------------------------------------------------
% Data:      data set
% omg:       the var-cov matrix of all moments at GMM-CUE estimator
% inv_omg:   inverse of omg cacluated by the explicit formula
% G:         Jacobian matrix of all moments at GMM-CUE estimator
% J:         J test statistic from GMM-CUE 
% m_theta0:  n*k matrix of the moments evaluated at GMM-CUE estimator
% m0_1:      n*l_grid matrix of the 1st baseline moment at grid points
% m0_2:      n*l_grid matrix of the 2nd baseline moment at grid points
% W0_theta:  l_grid*(k0^2) matrix of the weight matrix of basline moments
%--------------------------------------------------------------------------
n  = size(Data,1);      k0 = 2;   k1 = 1;  k  = k0 + k1;            

B  = 2500;              v  = mvnrnd(zeros(k,1), omg, B);
 
g_hat   = mean(m_theta0);                                       %  1 * 3

l_grid  = size(m0_1,2);
%--------------------------------------------------------------------------
%                     Calculate the conditional C.V.
%--------------------------------------------------------------------------
g0_theta = [mean(m0_1)', mean(m0_2)'];                          % l_grid*2

J0_theta = n*( kron(ones(1,k0),g0_theta).*W0_theta.*kron(g0_theta,ones(1,k0)) )*ones(k0^2,1); 

C        = J - min(J0_theta);

D        = eye(k) - G*pinv(G'*inv_omg *G)*G'*inv_omg;

m_theta  = sqrt(n)*(g0_theta - ones(l_grid,1)*g_hat(1,1:2));    % l_grid*2   Note: V(theta) = S_0 in this case

v_B      = D*v';                                                %  k * B

g0_theta_v1 = m_theta(:,1)*ones(1,B) + ones(l_grid,1)*v_B(1,:); % l_grid*B

g0_theta_v2 = m_theta(:,2)*ones(1,B) + ones(l_grid,1)*v_B(2,:); % l_grid*B

J0_theta_B  = (g0_theta_v1.*(W0_theta(:,1)*ones(1,B)) + g0_theta_v2.*(W0_theta(:,2)*ones(1,B))).*g0_theta_v1...
            + (g0_theta_v1.*(W0_theta(:,3)*ones(1,B)) + g0_theta_v2.*(W0_theta(:,4)*ones(1,B))).*g0_theta_v2;

J0_theta_B  = min(J0_theta_B);

J1_B        = ((v_B'*inv_omg).*v_B')*ones(k,1);

B_cv        = J1_B' - J0_theta_B;

pval        = mean((B_cv > C));
    
CC_cv       = quantile(B_cv, 0.95);
    
if C > CC_cv; test = 1; else, test = 0; end

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%        Function used to export quantities the GMM-CUE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 function [est_theta, omg, inv_omg, pval_J, G, J, m_theta0, m0_1, m0_2, W0_theta] = gmm_cue(Data, nuisance, theta_g)
%--------------------------------------------------------------------------
% Data:     data set
% nuisance: cailbrated values
% theta_g:  reparametrized grid points used to find the GMM-CUE
%--------------------------------------------------------------------------
k0 = 2;  k1 = 1;  k = k0 + k1;  d_theta = 1;

[est_theta, J, ~, omg, m_theta0, m0_1, m0_2, W0_theta] = cue_est(theta_g, Data, nuisance);

inv_omg  = inv_matrix(omg);

pval_J   = 1 - chi2cdf(J, k - d_theta);

G        = zeros(d_theta, k);

S        = eye(d_theta);

epsilon  = 0.00001;

for k = 1:d_theta
    
    d_theta1 = est_theta + epsilon*S(k,:);
    
    d_theta2 = est_theta - epsilon*S(k,:);
    
    [~, ~, gmm_d1, ~] = cue_est(d_theta1,Data,nuisance);
    
    [~, ~, gmm_d2, ~] = cue_est(d_theta2,Data,nuisance);
    
    G(k,:) = (gmm_d1 - gmm_d2)/(2*epsilon);
    
end

G = G';

 end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Function used to conduct full CUE-GMM estimation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [b, f_val, mean_g, omg, m_theta0, m0_1, m0_2, W0_theta] = cue_est(theta_g, Data, nuisance)
%--------------------------------------------------------------------------
% theta:    the parameter (the orginal one/the reparametrized one)
% Data:     The data set
% nuisance: the cailbrated values
%--------------------------------------------------------------------------
%                    Calibrated  Nuisance parameters
%--------------------------------------------------------------------------
sigma    = nuisance(1);     v  = nuisance(2);    

gamma    = nuisance(3);     p  = nuisance(4);  

sigma_d  = nuisance(5); 
%--------------------------------------------------------------------------
%                              Data
%--------------------------------------------------------------------------
d_c    = Data(:,1);         re = Data(:,2);        n = size(Data,1);
%--------------------------------------------------------------------------
%                        Reparametrization
%--------------------------------------------------------------------------
alpha  = gamma + p./theta_g;

l_grid = length(alpha);
%--------------------------------------------------------------------------
%                     Baseline Moment Conditions
%--------------------------------------------------------------------------
mu1    = v + 1./alpha;

mu2    = v^2 + 2*v./alpha     + 2./(alpha.^2);

mu3    = v^3 + 3*(v^2)./alpha + 6*v./(alpha.^2)      + 6./(alpha.^3);

mu4    = v^4 + 4*(v^3)./alpha + 12*(v^2)./(alpha.^2) + 24*v./(alpha.^3) + 24./(alpha.^4);

m0_1   = d_c*ones(1,l_grid) + p*ones(n,1)*mu1;

m0_2   = (d_c.^2)*ones(1,l_grid) - sigma^2 - p*ones(n,1)*mu2;
%--------------------------------------------------------------------------
%                  Asset Pricing Moment Conditions
%--------------------------------------------------------------------------
h_a    = alpha.*( exp(gamma*v) - (alpha-gamma)*exp((gamma-1)*v)./(alpha-gamma+1) );

mu     = gamma*sigma^2 - sigma^2/2 - p*mu1 + p*h_a./(alpha-gamma);

m1_1   = re*ones(1,l_grid) - ones(n,1)*mu;
%--------------------------------------------------------------------------
%     Variance-Covariance Matrix and Its Inverse omg^(-1) = W/D
%--------------------------------------------------------------------------
omg11  = sigma^2 + p*mu2 - (p*mu1).^2;
omg22  = 2*sigma^4 + 4*sigma^2*p*mu2 + p*mu4 - (p*mu2).^2;
omg33  = omg11 + sigma_d^2;
omg12  = 2*sigma^2*p*mu1 - p*mu3 + p^2*(mu1.*mu2);
omg13  = omg11;
omg23  = omg12;

W11    = omg33.*omg22 - omg23.^2;
W12    = omg13.*omg23 - omg33.*omg12;
W13    = omg12.*omg23 - omg13.*omg22;
W22    = omg33.*omg11 - omg13.^2;
W23    = omg12.*omg13 - omg11.*omg23;
W33    = omg11.*omg22 - omg12.^2;

D      = omg11.*W11 + omg12.*W12 + omg13.*W13;
%--------------------------------------------------------------------------
%                           Output 
%--------------------------------------------------------------------------
m_g = [mean(m0_1); mean(m0_2); mean(m1_1)];

f_0 = (m_g(1,:).*W11 + m_g(2,:).*W12 + m_g(3,:).*W13).*m_g(1,:)...
    + (m_g(1,:).*W12 + m_g(2,:).*W22 + m_g(3,:).*W23).*m_g(2,:)...
    + (m_g(1,:).*W13 + m_g(2,:).*W23 + m_g(3,:).*W33).*m_g(3,:);

f_1 = n*f_0./D;

[f_val,loc] = min(f_1);

b       = theta_g(loc);

mean_g  = m_g(:,loc)';

omg     = [omg11(loc), omg12(loc), omg13(loc);...
           omg12(loc), omg22(loc), omg23(loc);...
           omg13(loc), omg23(loc), omg33(loc)];
       
m_theta0 = [m0_1(:,loc), m0_2(:,loc), m1_1(:,loc)];       
       
D        = omg11.*omg22 - omg12.^2;

W11      = (  omg22./D )';

W22      = (  omg11./D )';

W12      = ( -omg12./D )';

W0_theta = [W11, W12, W12, W22];      
 
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Function used to calculate the inverse of 3*3 symmetric matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function f_val = inv_matrix(omg)

W11 = omg(3,3)*omg(2,2) - omg(2,3)^2;
W12 = omg(1,3)*omg(2,3) - omg(3,3)*omg(1,2);
W13 = omg(1,2)*omg(2,3) - omg(1,3)*omg(2,2);

D   = omg(1,1)*W11 + omg(1,2)*W12 + omg(1,3)*W13;

W11 = W11/D;
W12 = W12/D;
W13 = W13/D;

W22 = (omg(3,3)*omg(1,1) - omg(1,3)^2)/D;
W23 = (omg(1,2)*omg(1,3) - omg(1,1)*omg(2,3))/D;
W33 = (omg(1,1)*omg(2,2) - omg(1,2)^2)/D;

f_val = [W11, W12, W13; W12, W22, W23; W13, W23, W33];

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%             Function used to generate data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function data = SData(nuisance, para, n)

theta   = para(1);       eta   =   para(2);

sigma   = nuisance(1);   v     = nuisance(2);    

gamma   = nuisance(3);   p     = nuisance(4);

sigr    = nuisance(5);

alpha   = p/theta + gamma;  

eps     = normrnd(0,1,n,2);    %errors

J       = exprnd(1/alpha,n,1);

x       = binornd(1,p,n,1);

delta_c = sigma*eps(:,1) - x.*(v+J);

mu_1    = v + 1/alpha;

h_alpha = alpha*( exp(gamma*v) - (alpha-gamma)*exp((gamma-1)*v)/(alpha-gamma+1) );

mu      = eta + gamma*sigma^2 - sigma^2/2 - p*mu_1 + p*h_alpha/(alpha-gamma);

e_dt    = delta_c + p*mu_1 + sigr*eps(:,2);

re      = mu + e_dt;

data    = [delta_c, re];

end