clear; clc; n = 500; S = 10000; M = 80; L = 10;

cr_v_J = chi2inv(0.95 , 2);  cr_v_C = chi2inv(0.95 , 1); 

sig_c = 0.0072 * sqrt(3);      % volatility of consumption growth   
delta = 0.9989^3;              % subjective discount factor 
gamma = 10;                    % risk aversion
psi   = 1.5;                   % EIS
rho   = 0.975^3;               % persistence of long-run risk 
sig_l = 7.5/100;               % additional volatility of stock excess returns 
sig_x = 1;                     % long-run component volatility 

solvetheta = @(tt)(gamma*sig_c^2 - 1/2*sig_c^2 + 1/2*(2*gamma-1/psi-1)*(1-1/psi)* tt^2);           
theta0     = abs(fzero(@(tt)(solvetheta(tt) - 1.5/100), 0.07));
phi        = theta0*( 1/delta - rho ); 

nuisance   = [sig_c  delta  gamma  psi  phi  sig_l  sig_x];

theta1_1   = abs(fzero(@(tt)(solvetheta(tt) - 0.75/100),  0.07));
theta1_2   = abs(fzero(@(tt)(solvetheta(tt) - 2.25/100),  0.07));
theta2_1   = abs(fzero(@(tt)(solvetheta(tt) - 0.50/100),  0.07));
theta2_2   = abs(fzero(@(tt)(solvetheta(tt) - 2.50/100),  0.07));

theta_spac = [theta1_1, theta1_2; theta2_1, theta2_2];

eta        = linspace(-0.03, 0.03, M)';
eta        = [eta(1:M/2);0;eta(M/2+1:M)];
M          = M + 1;
para       = [theta0*ones(M,1), eta];

for  b  = [1, 2]

tic;      
    
tseed   = 0;             randn('seed', tseed);

J_test  = zeros(M,1);    C_test   = zeros(M,1);   CC_test   = zeros(M,1);

theta_g = linspace(theta_spac(b,1), theta_spac(b,2), 2000);

  for  m  = 1:M
    
  para_m  = para(m,:);
    
  J_test_m = zeros(S,1);   C_test_m = zeros(S,1);   CC_test_m = zeros(S,1);

    parfor s = 1:S

    Data = SData(nuisance, para_m, n); 

    [est_theta, omg, inv_omg, pval_J, G, J, mean_g, dc01_n, dc02_n, dc12_n, dc11_n, rho_g, g0_theta, W0_theta] = gmm_cue(Data, nuisance, theta_g, L);

    [pval, CC_test_rst, C] = conditional_inference(Data, omg, inv_omg, G, J, L, mean_g, dc01_n, dc02_n, dc12_n, dc11_n, rho_g, g0_theta, W0_theta);

    if J > cr_v_J; J_test_m(s,1) = 1; end

    if C > cr_v_C; C_test_m(s,1) = 1; end

    CC_test_m(s,1) = CC_test_rst;

    end
    
    J_test(m,1) = mean(J_test_m); C_test(m,1) = mean(C_test_m); CC_test(m,1) = mean(CC_test_m);
    
  end

time_cost   = toc;

name = 'LRR_J_and_CC_%d.mat';

save(sprintf(name,b),'J_test','C_test','CC_test', 'time_cost');  

end


% Auxiliary functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%        Function used to conduct the conditional specification test
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [pval, test, C] = conditional_inference(Data, omg, inv_omg, G, J, L, m_theta0, dc01_n, dc02_n, dc12_n, dc11_n, rho_g, g0_theta, W0_theta)
%--------------------------------------------------------------------------
% Data:      data set
% omg:       the variance-covariance matrix of all moments at "cue_est"
% inv_omg:   inverse of omg cacluated by the explicit formula
% G:         Jacobian matrix of all moments at GMM-CUE estimator
% J:         J test statistic from GMM-CUE 
% L:         number of lags in Newey-West
% m_theta0:  n*k matrix of the moments evaluated at GMM-CUE estimator
% dc01_n:    ( dc_{t-1} - E_n[dc_{t-1}] )*( dc_{t} - E_n[dc_{t}] )
% dc02_n:    ( dc_{t-1} - E_n[dc_{t-1}] )*( dc_{t+1} - E_n[dc_{t+1}] )
% dc12_n:    ( dc_{t} - E_n[dc_{t}] )*( dc_{t+1} - E_n[dc_{t+1}] )
% dc11_n:    ( dc_{t} - E_n[dc_{t}] )*( dc_{t} - E_n[dc_{t}] )
% rho_g:     grid points of rho
% g0_theta:  l_grid*k0 matrix of mean of baseline moments at grid points    
% W0_theta:  k0^2 * l_grid matrix of the weight matrix of basline moments
%--------------------------------------------------------------------------
n  = size(Data,1);      k0 = 2;   k1 = 1;  k  = k0 + k1;            

B  = 2500;              v  = mvnrnd(zeros(k,1), omg, B);

mean_g     = mean(m_theta0);                          % 1 * k
%--------------------------------------------------------------------------
%               Calculating the V(theta) matrix
%--------------------------------------------------------------------------
m_theta0_n = (m_theta0 - ones(n,1)*mean_g)*inv_omg;   % n * k

V1         = [dc02_n, dc01_n]'*m_theta0_n;            % k0 * K
V2         = [dc12_n, dc11_n]'*m_theta0_n;            % k0 * K

for m   = 1:L

  wei   = 1 - m/(L+1);

  V1    = V1 + wei*( [dc02_n(1:n-m,:), dc01_n(1:n-m,:)]'*m_theta0_n(m+1:n,:)... 
            +        [dc02_n(m+1:n,:), dc01_n(m+1:n,:)]'*m_theta0_n(1:n-m,:) );
         
  V2    = V2 + wei*( [dc12_n(1:n-m,:), dc11_n(1:n-m,:)]'*m_theta0_n(m+1:n,:)... 
            +        [dc12_n(m+1:n,:), dc11_n(m+1:n,:)]'*m_theta0_n(1:n-m,:) );

end

V11    = V1(1,1) - V1(2,1)*rho_g';
V12    = V1(1,2) - V1(2,2)*rho_g';
V13    = V1(1,3) - V1(2,3)*rho_g';
V21    = V2(1,1) - V2(2,1)*rho_g';
V22    = V2(1,2) - V2(2,2)*rho_g';
V23    = V2(1,3) - V2(2,3)*rho_g';

V_theta = [V11, V12, V13, V21, V22, V23]/n;                                  % l_grid * 6
%--------------------------------------------------------------------------
%                     Calculate the conditional C.V.
%--------------------------------------------------------------------------
J0_theta = n*(kron(ones(1,k0),g0_theta).*W0_theta.*kron(g0_theta,ones(1,k0)))*ones(k0^2,1); 

C        = J - min(J0_theta);

D        = eye(k) - G*pinv(G'*inv_omg *G)*G'*inv_omg;

m_theta  = sqrt(n)*(g0_theta - (V_theta*kron(eye(k0),mean_g')));           % l_grid * 2

v_B      = D*v';                                                           % k * B

g0_theta_v1 = m_theta(:,1)*ones(1,B) + V_theta(:,1:3)*v_B;                 % l_grid * B

g0_theta_v2 = m_theta(:,2)*ones(1,B) + V_theta(:,4:6)*v_B;                 % l_grid * B


J0_theta_B  = (g0_theta_v1.*(W0_theta(:,1)*ones(1,B)) + g0_theta_v2.*(W0_theta(:,2)*ones(1,B))).*g0_theta_v1...
            + (g0_theta_v1.*(W0_theta(:,3)*ones(1,B)) + g0_theta_v2.*(W0_theta(:,4)*ones(1,B))).*g0_theta_v2;

J0_theta_B  = min(J0_theta_B);

J1_B        = ((v_B'*inv_omg).*v_B')*ones(k,1);

B_cv        = J1_B' - J0_theta_B;

pval        = mean((B_cv > C));
    
CC_cv       = quantile(B_cv,0.95);
    
if C > CC_cv; test = 1; else, test = 0; end

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%        Function used to export quantities the GMM-CUE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 function [est_theta, omg, inv_omg, pval_J, G, J, m_theta0, dc01_n, dc02_n, dc12_n, dc11_n, rho_g, g0_theta, W0_theta] = gmm_cue(Data, nuisance, theta_g, L)
%--------------------------------------------------------------------------
% Data:      data set
% nuisance:  cailbrated values
% theta_g:   reparametrized grid points used to find the GMM-CUE
% L:         number of lags in Newey-West
%--------------------------------------------------------------------------
k0 = 2;  k1 = 1;  k = k0 + k1;  d_theta = 1;

[est_theta, J, ~, omg, m_theta0, dc01_n, dc02_n, dc12_n, dc11_n, rho_g, g0_theta, W0_theta] = cue_est(theta_g, Data, nuisance, L);

inv_omg  = inv_matrix(omg);

pval_J   = 1 - chi2cdf(J, k - d_theta);

G        = zeros(d_theta, k);

S_L      = eye(d_theta);

epsilon  = 0.00001;

for k = 1:d_theta
    
    d_theta1 = est_theta + epsilon*S_L(k,:);
    
    d_theta2 = est_theta - epsilon*S_L(k,:);
    
    [~, ~, gmm_d1] = cue_est(d_theta1,Data,nuisance, L);
    
    [~, ~, gmm_d2] = cue_est(d_theta2,Data,nuisance, L);
    
    G(k,:) = (gmm_d1 - gmm_d2)/(2*epsilon);
    
end

G = G';

 end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Function used to calculate the full CUE-GMM criterion value
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [b, f_val, mean_g, omg, m_theta0, dc01_n, dc02_n, dc12_n, dc11_n, rho_g, g0_theta, W0_theta] = cue_est(theta_g, Data, nuisance, L)
%--------------------------------------------------------------------------
% theta_g:   the reparametrized parameter 
% Data:      data set
% nuisance:  cailbrated values
% L:         number of lags in Newey-West
%--------------------------------------------------------------------------
%                    Calibrated  Nuisance parameters
%--------------------------------------------------------------------------
sig_c = nuisance(1);     delta = nuisance(2);    

gamma = nuisance(3);     psi   = nuisance(4);

phi   = nuisance(5);     
%--------------------------------------------------------------------------
%                              Data
%--------------------------------------------------------------------------
dc0 = Data(:,1);         dc1   = Data(:,2);        dc2 = Data(:,3);

re  = Data(:,4);         n     = size(Data,1);
%--------------------------------------------------------------------------
%                        Reparametrization
%--------------------------------------------------------------------------
rho_g  = 1/delta - phi./theta_g; 

l_grid = length(rho_g);
%--------------------------------------------------------------------------
%                     Baseline Moment Conditions
%--------------------------------------------------------------------------
dc01   = dc0.*dc1;      dc02   = dc0.*dc2;       

dc12   = dc1.*dc2;      dc11   = dc1.*dc1;       

m0_1   =  dc02*ones(1,l_grid) - dc01*rho_g;

m0_2   =  dc12*ones(1,l_grid) - dc11*rho_g + (sig_c^2)*ones(n,1)*rho_g;
%--------------------------------------------------------------------------
%                   Asset Pricing Moment Conditions
%--------------------------------------------------------------------------
mu     = gamma*sig_c^2 - sig_c^2/2 + (2*gamma-1/psi-1)*(1-1/psi)*(theta_g.^2)/2;

m1_1   = re*ones(1,l_grid) - ones(n,1)*mu;
%--------------------------------------------------------------------------
%        Calculate the Var-cov Matrix and Its Inverse omg^(-1) = W/D
%--------------------------------------------------------------------------
dc01_n = dc01 - mean(dc01);    dc02_n = dc02 - mean(dc02);
dc12_n = dc12 - mean(dc12);    dc11_n = dc11 - mean(dc11);
re_n   = re - mean(re);

Domg11 = [dc02_n'*dc02_n, dc02_n'*dc01_n, dc01_n'*dc02_n, dc01_n'*dc01_n];
Domg22 = [dc12_n'*dc12_n, dc12_n'*dc11_n, dc11_n'*dc12_n, dc11_n'*dc11_n];
Domg33 = re_n'*re_n;
Domg12 = [dc02_n'*dc12_n, dc02_n'*dc11_n, dc01_n'*dc12_n, dc01_n'*dc11_n];
Domg13 = [dc02_n'*re_n, dc01_n'*re_n];
Domg23 = [dc12_n'*re_n, dc11_n'*re_n];

for  m  = 1:L
    
 wei    = 1 - m/(L+1);
  
 Domg11 = Domg11 + 2*wei*[dc02_n(1:n-m,:)'*dc02_n(m+1:n,:), dc02_n(1:n-m,:)'*dc01_n(m+1:n,:),... 
                          dc01_n(1:n-m,:)'*dc02_n(m+1:n,:), dc01_n(1:n-m,:)'*dc01_n(m+1:n,:)];
                    
 Domg22 = Domg22 + 2*wei*[dc12_n(1:n-m,:)'*dc12_n(m+1:n,:), dc12_n(1:n-m,:)'*dc11_n(m+1:n,:),... 
                          dc11_n(1:n-m,:)'*dc12_n(m+1:n,:), dc11_n(1:n-m,:)'*dc11_n(m+1:n,:)];
 
 Domg33 = Domg33 + 2*wei*re_n(1:n-m,:)'*re_n(m+1:n,:);
 
 Domg12 = Domg12 +   wei*[dc02_n(1:n-m,:)'*dc12_n(m+1:n,:), dc02_n(1:n-m,:)'*dc11_n(m+1:n,:),... 
                          dc01_n(1:n-m,:)'*dc12_n(m+1:n,:), dc01_n(1:n-m,:)'*dc11_n(m+1:n,:)]...
                 +   wei*[dc02_n(m+1:n,:)'*dc12_n(1:n-m,:), dc02_n(m+1:n,:)'*dc11_n(1:n-m,:),... 
                          dc01_n(m+1:n,:)'*dc12_n(1:n-m,:), dc01_n(m+1:n,:)'*dc11_n(1:n-m,:)];
                      
 Domg13 = Domg13 +   wei*[dc02_n(1:n-m,:)'*re_n(m+1:n,:),   dc01_n(1:n-m,:)'*re_n(m+1:n,:)]...    
                 +   wei*[dc02_n(m+1:n,:)'*re_n(1:n-m,:),   dc01_n(m+1:n,:)'*re_n(1:n-m,:)];       
             
 Domg23 = Domg23 +   wei*[dc12_n(1:n-m,:)'*re_n(m+1:n,:),   dc11_n(1:n-m,:)'*re_n(m+1:n,:)]...
                 +   wei*[dc12_n(m+1:n,:)'*re_n(1:n-m,:),   dc11_n(m+1:n,:)'*re_n(1:n-m,:)];
                         
end

omg11  = (Domg11(1) - Domg11(2)*rho_g - Domg11(3)*rho_g +  Domg11(4)*rho_g.^2)/n;

omg22  = (Domg22(1) - Domg22(2)*rho_g - Domg22(3)*rho_g +  Domg22(4)*rho_g.^2)/n;

omg33  = (Domg33*ones(1,l_grid))/n;

omg12  = (Domg12(1) - Domg12(2)*rho_g - Domg12(3)*rho_g +  Domg12(4)*rho_g.^2)/n;

omg13  = (Domg13(1) - Domg13(2)*rho_g)/n;

omg23  = (Domg23(1) - Domg23(2)*rho_g)/n;

W11    = omg33.*omg22 - omg23.^2;
W12    = omg13.*omg23 - omg33.*omg12;
W13    = omg12.*omg23 - omg13.*omg22;
W22    = omg33.*omg11 - omg13.^2;
W23    = omg12.*omg13 - omg11.*omg23;
W33    = omg11.*omg22 - omg12.^2;

D      = omg11.*W11 + omg12.*W12 + omg13.*W13;
%--------------------------------------------------------------------------
%                               Output
%--------------------------------------------------------------------------
m_g    = [mean(m0_1); mean(m0_2); mean(m1_1)];

f_0    = (m_g(1,:).*W11 + m_g(2,:).*W12 + m_g(3,:).*W13).*m_g(1,:)...
       + (m_g(1,:).*W12 + m_g(2,:).*W22 + m_g(3,:).*W23).*m_g(2,:)...
       + (m_g(1,:).*W13 + m_g(2,:).*W23 + m_g(3,:).*W33).*m_g(3,:);

f_1    = n*f_0./D;

[f_val,loc] = min(f_1);

b      = theta_g(loc);

mean_g = m_g(:,loc)';

omg    = [omg11(loc), omg12(loc), omg13(loc);...
          omg12(loc), omg22(loc), omg23(loc);...
          omg13(loc), omg23(loc), omg33(loc)];
        
m_theta0 = [m0_1(:,loc), m0_2(:,loc), m1_1(:,loc)];    

g0_theta = [mean(m0_1)', mean(m0_2)'];              % l_grid * 2
       
D        = omg11.*omg22 - omg12.^2;

W11      = (  omg22./D )';

W22      = (  omg11./D )';

W12      = ( -omg12./D )';

W0_theta = [W11, W12, W12, W22];      
       
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Function used to calculate the inverse of 3*3 symmetric matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function f_val = inv_matrix(omg)

W11 = omg(3,3)*omg(2,2) - omg(2,3)^2;
W12 = omg(1,3)*omg(2,3) - omg(3,3)*omg(1,2);
W13 = omg(1,2)*omg(2,3) - omg(1,3)*omg(2,2);

D   = omg(1,1)*W11 + omg(1,2)*W12 + omg(1,3)*W13;

W11 = W11/D;
W12 = W12/D;
W13 = W13/D;

W22 = (omg(3,3)*omg(1,1) - omg(1,3)^2)/D;
W23 = (omg(1,2)*omg(1,3) - omg(1,1)*omg(2,3))/D;
W33 = (omg(1,1)*omg(2,2) - omg(1,2)^2)/D;

f_val = [W11, W12, W13; W12, W22, W23; W13, W23, W33];

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Function used to generate data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function data = SData(nuisance, para, n) 

theta = para(1);         eta   =   para(2);

sig_c = nuisance(1);     delta = nuisance(2);    

gamma = nuisance(3);     psi   = nuisance(4);

phi   = nuisance(5);     sig_l = nuisance(6);

sig_x = nuisance(7);

rho   = 1/delta - phi/theta; 

T     = n + 100;           % burn-in sample = 100

eps   = normrnd(0,1,T,3);  % errors

eps_x = eps(:,1);

eps_c = eps(:,2);

eps_l = eps(:,3);

x     = zeros(T,1);

x(1)  = normrnd(0, sig_x^2/(1-rho^2), 1, 1); % x0 from unconditional dist

for t = 2:T
    
    x(t) = rho*x(t-1) + sig_x*eps_x(t);

end

delta_c = phi*x(1:T-1) + sig_c*eps_c(2:T);   % delta_c is (T-1)*1

mu      = eta + gamma*sig_c^2 - sig_c^2/2 + (2*gamma-1/psi-1)*(1-1/psi)*theta^2/2;

re      = mu + sig_c*eps_c(2:T) + (1-1/psi)*theta*eps_x(2:T) + sig_l*eps_l(2:T); % re is (T-1)*1

delta_c = delta_c(T-n-2:T-1,1); % extract the last n+2 subvector of delta_c

re      = re(T-n-2:T-1,1);      % extract the last n+2 subvector of re

data    = [delta_c(1:n,1) delta_c(2:n+1,1) delta_c(3:n+2,1) re(2:n+1,1)];  %[dc_{t-1}, dc_{t}, dc_{t+1}, re_{t}]

end