clear; close all; clc;

rng('default');

S = 10000; B = 2500;       k1 = 2;   rho_G = 0.1; 

Theta  = -1:0.001:1;       l_Theta = length(Theta);

grid_a = -10:1:10;         l_a     = length(grid_a);

G1 = zeros(k1,1); for j = 1:k1;  G1(j) = 1/j; end

S1 = [0.0, 0.4, 1, 1; 0.0, 0.4, 2, 1;... 
      0.0, 0.2, 2, 1; 0.0, 0.8, 2, 1;... 
      0.1, 0.4, 1, 1; 0.1, 0.4, 2, 1];

S2 = [0.0, 0.4, 1, 1; 0.0, 0.4, 2, 1];

for DGP = [1, 2]
    
if DGP == 1; Set = S1; elseif DGP == 2; Set = S2; end
    
l_S = size(Set,1);    
    
for l = 1:l_S
    
    l
    
 c_g = Set(l, 1); rho = Set(l, 2); q = Set(l, 3); c_mu = Set(l, 4);

 k0  = q*k1; k = k0 + k1;

 cv1 = chi2inv(0.95, k-1);
 
 o_u = [1 rho_G;rho_G 1];
 
 Omg = [(eye(k0) + rho^2*kron(ones(q,q),o_u))/(1+rho^2), rho*kron(ones(q,1),o_u); rho*kron(ones(1,q),o_u) o_u];
    
 if DGP == 1
    
 Sig = [Omg zeros(k,k0); zeros(k0,k+k0)]; 
    
 elseif DGP == 2
     
 B1  = rho_G*[eye(k0); kron(ones(1,q),eye(k1))];
        
 Sig = [Omg B1; B1' eye(k0)];
       
 end       
 
 G     = [c_g*ones(k0,1); G1];      G_orth  = null(G');

 D     = eye(k)  - G*G'*pinv(Omg)/(G'*pinv(Omg)*G);

 Omg_R = G_orth'*Omg*G_orth;

 [V_theta,inv_Omg_theta] = nuisance_fun(Sig, Theta, k0, k1);

 emp_rej = zeros(l_a, 4);
    
 for m = 1:l_a

 a        = grid_a(m);    detal = a*ones(k1,1);

 mu       = [zeros(k0,1); detal; c_mu*ones(k0,1)];

 R_mu     = G_orth'*mu(1:k,1);

 emp_rej1 = zeros(S,1);   emp_rej2 = zeros(S,1);

 emp_rej3 = zeros(S,1);   emp_rej4 = zeros(S,1);

 parfor s = 1:S
    
    v  = mvnrnd(zeros(k,1),Omg,B);
     
    X  = mvnrnd(mu,Sig,1); 
    
    g_inf  = X(1,1:k)';  g0_inf = X(1,1:k0)';     G0_inf = X(k+1:k+k0)';
    
    Y  = D*g_inf;        Y_star = G_orth'*g_inf;
    
    g0_theta = ones(l_Theta,1)*g0_inf' + Theta'*G0_inf';
    
    J0_theta = ((kron(ones(1,k0),g0_theta).*inv_Omg_theta).*kron(g0_theta,ones(1,k0)))*ones(k0^2,1);
    
    C        = Y'*pinv(Omg)*Y - min(J0_theta);
    
    m_theta  = g0_theta - (V_theta*kron(eye(k0),Y));

    B_cv    = zeros(B,1);

    for b = 1:B

        v_b = D*v(b,:)';
        
        g0_theta_v = m_theta + V_theta*kron(eye(k0), v_b);
        
        J0_theta_b = ((kron(ones(1,k0),g0_theta_v).*inv_Omg_theta).*kron(g0_theta_v,ones(1,k0)))*ones(k0^2,1);

        C_b        = v_b'*pinv(Omg)*v_b - min(J0_theta_b);

        B_cv(b)    = C_b;

    end

    C_cv  = quantile(B_cv,0.95);

    if C  > C_cv; emp_rej1(s,1) = 1; end
    
    if a == 0; stud_t = mvnrnd(0,1,1); 
    
    else
        
        stud_t = Y_star'*pinv(Omg_R)*R_mu/sqrt(R_mu'*pinv(Omg_R)*R_mu);

    end
    
    if stud_t > 1.6449;    emp_rej2(s,1) = 1; end
    
    if abs(stud_t) > 1.96; emp_rej3(s,1) = 1; end
    
    if Y_star'*pinv(Omg_R)*Y_star > cv1; emp_rej4(s,1) = 1; end
    
 end

 emp_rej(m,:) = [mean(emp_rej1), mean(emp_rej2), mean(emp_rej3), mean(emp_rej4)];

 end

 name = 'PE_WeakID_%d_%d_%d_%d_%d.mat';

 save(sprintf(name,DGP,c_g*100,rho*100,q,c_mu),'emp_rej');  
    
end
    
end

%--------------------------------------------------------------------------
% Function used to calculate V(theta) and the inverse of \Omega_0(theta) 
%--------------------------------------------------------------------------
function [f1, f2] = nuisance_fun(Sig, Theta, k0, k1) 
%--------------------------------------------------------------------------
% Sig:   covariance matrix of (g_infinity(theta_0)', G_{0,infinity}')'
% Theta: grid points of \theta in the parameter space
% k0:    the number of the baseline moment conditions
% k1:    the number of the asset pricing moment conditions
%--------------------------------------------------------------------------
k      = k0 + k1;

l_grid = length(Theta);

Omg_00 = Sig(1:k0,1:k0);

Omg_01 = Sig(1:k0,k0+1:k);

Omg_g0 = Sig(k+1:k+k0,1:k0);

Omg_g1 = Sig(k+1:k+k0,k0+1:k);

Omg_gg = Sig(k+1:k+k0,k+1:k+k0);

V1     = [Omg_00, Omg_01]*pinv(Sig(1:k, 1:k));

V2     = [Omg_g0, Omg_g1]*pinv(Sig(1:k, 1:k));

f1     = ones(l_grid,1)*reshape(V1',1,[]) + Theta'*reshape(V2',1,[]);

f2     = zeros(l_grid, k0^2);

 for m = 1:l_grid
    
  inv_Omg_0 = pinv(Omg_00 + Theta(m)*(Omg_g0 + Omg_g0') + Theta(m)^2*Omg_gg);
    
  f2(m,:)   = reshape(inv_Omg_0, 1, []);

 end  

end
 